@@ -1,24 +1,27 @@
-/*
-
-Copyright (C) 2011-2019 Michael Goffioul
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 2011-2021 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -30,27 +33,35 @@ along with Octave; see the file COPYING.  If not, see
 #include "PopupMenuControl.h"
 #include "QtHandlesUtils.h"
 
+#include "octave-qobject.h"
+
 namespace QtHandles
 {
 
   PopupMenuControl*
-  PopupMenuControl::create (const graphics_object& go)
+  PopupMenuControl::create (octave::base_qobject& oct_qobj,
+                            octave::interpreter& interp,
+                            const graphics_object& go)
   {
-    Object *parent = Object::parentObject (go);
+    Object *parent = parentObject (interp, go);
 
     if (parent)
       {
         Container *container = parent->innerContainer ();
 
         if (container)
-          return new PopupMenuControl (go, new QComboBox (container));
+          return new PopupMenuControl (oct_qobj, interp, go,
+                                       new QComboBox (container));
       }
 
     return nullptr;
   }
 
-  PopupMenuControl::PopupMenuControl (const graphics_object& go, QComboBox *box)
-    : BaseControl (go, box), m_blockUpdate (false)
+  PopupMenuControl::PopupMenuControl (octave::base_qobject& oct_qobj,
+                                      octave::interpreter& interp,
+                                      const graphics_object& go,
+                                      QComboBox *box)
+    : BaseControl (oct_qobj, interp, go, box), m_blockUpdate (false)
   {
     uicontrol::properties& up = properties<uicontrol> ();
 
@@ -88,10 +99,9 @@ namespace QtHandles
             }
           else
             {
-              gh_manager::post_set (m_handle, "value",
-                                    octave_value (box->count () > 0
-                                                  ? 1.0 : 0.0),
-                                    false);
+              emit gh_set_event (m_handle, "value",
+                                 octave_value (box->count () > 0 ? 1.0 : 0.0),
+                                 false);
             }
         }
         m_blockUpdate = false;
@@ -134,10 +144,9 @@ namespace QtHandles
   {
     if (! m_blockUpdate)
       {
-        gh_manager::post_set (m_handle, "value",
-                              octave_value (double (index + 1)),
-                              false);
-        gh_manager::post_callback (m_handle, "callback");
+        emit gh_set_event (m_handle, "value", octave_value (double (index + 1)),
+                           false);
+        emit gh_callback_event (m_handle, "callback");
       }
   }
 
