@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2018-2020 The Octave Project Developers
+// Copyright (C) 2018-2021 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -36,6 +36,10 @@
 #include <QFileInfo>
 #include <QHelpContentWidget>
 #include <QHelpIndexWidget>
+#if defined (HAVE_NEW_QHELPINDEXWIDGET_API) \
+  || defined (HAVE_QHELPENGINE_DOCUMENTSFORIDENTIFIER)
+#  include <QHelpLink>
+#endif
 #include <QHelpSearchEngine>
 #include <QHelpSearchQueryWidget>
 #include <QHelpSearchResultWidget>
@@ -182,6 +186,10 @@ namespace octave
 
     if (m_help_engine)
       {
+#if defined (HAVE_NEW_QHELPINDEXWIDGET_API)
+        // Starting in Qt 5.15, help engine uses filters instead of old API
+        m_help_engine->setUsesFilterEngine (true);
+#endif
         // Layout contents, index and search
         QTabWidget *navi = new QTabWidget (this);
         navi->setTabsClosable (false);
@@ -227,10 +235,17 @@ namespace octave
 
         navi->addTab (index_all, tr ("Function Index"));
 
-        connect(m_help_engine->indexWidget (),
-                SIGNAL (linkActivated (const QUrl&, const QString&)),
-                m_doc_browser, SLOT(handle_index_clicked (const QUrl&,
-                                                          const QString&)));
+#if defined (HAVE_NEW_QHELPINDEXWIDGET_API)
+        connect (m_help_engine->indexWidget (),
+                 &QHelpIndexWidget::documentActivated,
+                 this, [this](const QHelpLink &link) {
+                        m_doc_browser->handle_index_clicked (link.url);});
+#else
+        connect (m_help_engine->indexWidget (),
+                 SIGNAL (linkActivated (const QUrl&, const QString&)),
+                 m_doc_browser, SLOT(handle_index_clicked (const QUrl&,
+                                                           const QString&)));
+#endif
 
         connect (m_filter, SIGNAL (editTextChanged (const QString&)),
                  this, SLOT(filter_update (const QString&)));
@@ -429,7 +444,11 @@ namespace octave
     if (rx.indexIn (query_string, 0) != -1)
       m_internal_search = rx.cap (1);
     else
+#if defined (HAVE_QT_SPLITBEHAVIOR_ENUM)
+      m_internal_search = query_string.split (" ", Qt::SkipEmptyParts).first ();
+#else
       m_internal_search = query_string.split (" ", QString::SkipEmptyParts).first ();
+#endif
 
     m_help_engine->searchEngine ()->search (queries);
   }
@@ -613,15 +632,26 @@ namespace octave
     if (! m_help_engine || ref_name.isEmpty ())
       return;
 
-    // First search in the function index
+#if defined (HAVE_QHELPENGINE_DOCUMENTSFORIDENTIFIER)
+    QList<QHelpLink> found_links
+      = m_help_engine->documentsForIdentifier (ref_name);
+#else
     QMap<QString, QUrl> found_links
       = m_help_engine->linksForIdentifier (ref_name);
+#endif
 
     QTabWidget *navi = static_cast<QTabWidget*> (widget (0));
 
     if (found_links.count() > 0)
       {
-        m_doc_browser->setSource (found_links.constBegin().value());
+        // First search in the function index
+#if defined (HAVE_QHELPENGINE_DOCUMENTSFORIDENTIFIER)
+        QUrl first_url = found_links.constFirst().url;
+#else
+        QUrl first_url = found_links.constBegin().value ();
+#endif
+
+        m_doc_browser->setSource (first_url);
 
         // Switch to function index tab
         m_help_engine->indexWidget()->filterIndices (ref_name);
@@ -707,7 +737,7 @@ namespace octave
     if (! m_help_engine)
       return;
 
-    QTextDocument::FindFlags find_flags = 0;
+    QTextDocument::FindFlags find_flags;
     if (backward)
       find_flags = QTextDocument::FindBackward;
 
@@ -955,7 +985,11 @@ namespace octave
   {
     if (we->modifiers () == Qt::ControlModifier)
       {
-        if (we->delta () > 0)
+#if defined (HAVE_QWHEELEVENT_ANGLEDELTA)
+        if (we->angleDelta().y () > 0)
+#else
+        if (we->delta() > 0)
+#endif
           zoom_in ();
         else
           zoom_out ();
