@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2017-2020 The Octave Project Developers
+// Copyright (C) 2017-2021 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -59,8 +59,10 @@ namespace octave
     editor.replace ("%f", file);
     editor.replace ("%l", QString::number (line));
 
+    QStringList arguments;
+
     // start the process and check for success
-    bool started_ok = QProcess::startDetached (editor);
+    bool started_ok = QProcess::startDetached (editor, arguments);
 
     if (started_ok != true)
       {
