@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2005-2020 The Octave Project Developers
+// Copyright (C) 2005-2021 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -80,19 +80,20 @@ get_output_list (octave::error_system& es,
     {
       tmp = octave::feval (func, inputlist, nargout);
     }
-  catch (const octave::execution_exception& e)
+  catch (const octave::execution_exception& ee)
     {
       if (error_handler.is_defined ())
         {
           octave::interpreter& interp
             = octave::__get_interpreter__ ("get_output_list");
 
+          es.save_exception (ee);
           interp.recover_from_exception ();
 
           execution_error = true;
         }
       else
-        throw e;
+        throw;
     }
 
   if (execution_error)
@@ -545,6 +546,7 @@ nevermind:
 
       OCTAVE_LOCAL_BUFFER (octave_value, retv, nargout1);
 
+      int expected_nargout;
       for (octave_idx_type count = 0; count < k; count++)
         {
           for (int j = 0; j < nargin; j++)
@@ -557,12 +559,17 @@ nevermind:
             = get_output_list (es, count, nargout, inputlist, func,
                                error_handler);
 
-          if (nargout > 0 && tmp.length () < nargout)
+          int tmp_numel = tmp.length ();
+          if (count == 0)
+            expected_nargout = tmp_numel;
+          else if (tmp_numel != expected_nargout)
+            error ("cellfun: function returned unexpected number of values");
+
+          if (nargout > 0 && tmp_numel < nargout)
             error ("cellfun: function returned fewer than nargout values");
 
           if (nargout > 0
-              || (nargout == 0
-                  && tmp.length () > 0 && tmp(0).is_defined ()))
+              || (nargout == 0 && tmp_numel > 0 && tmp(0).is_defined ()))
             {
               int num_to_copy = tmp.length ();
 
@@ -680,6 +687,12 @@ nevermind:
 %!  ## Empty function
 %!endfunction
 
+%!function varargout = __f02 (out)
+%!  if (out)
+%!    varargout{1} = out;
+%!  endif
+%!endfunction
+
 %!test
 %! __cellfun_test_num_outputs__ = -1;
 %!
@@ -704,6 +717,10 @@ nevermind:
 %! assert (__cellfun_test_num_outputs__, 0);
 
 %!error x = cellfun (@__f01, {1, 2})
+%!error x = cellfun (@__f01, {1, 2})
+
+%!error x = cellfun (@__f02, {0, 2}, "uniformoutput", false)
+%!error x = cellfun (@__f02, {0, 2}, "uniformoutput", true)
 
 %!test
 %! assert (cellfun (@__f11, {1, 2}), [1, 2]);
@@ -987,6 +1004,17 @@ nevermind:
 %!error cellfun (@sin, {[]}, "BadParam", false)
 %!error cellfun (@sin, {[]}, "UniformOuput")
 %!error cellfun (@sin, {[]}, "ErrorHandler")
+
+%!function retval = __errfcn (S, varargin)
+%!  global __errmsg;
+%!  __errmsg = S.message;
+%!  retval = NaN;
+%!endfunction
+%!test <58411>
+%! global __errmsg;
+%! assert (cellfun (@factorial, {1, 2, -3}, "ErrorHandler", @__errfcn), [1, 2, NaN]);
+%! assert (! isempty (__errmsg));
+%! clear -global __errmsg;
 */
 
 // Arrayfun was originally a .m file written by Bill Denney and Jaroslav
