@@ -1,26 +1,27 @@
-/*
-
-Copyright (C) 1996-2019 John W. Eaton
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
-
-// Author: John W. Eaton.
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1996-2021 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -51,6 +52,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "defun.h"
 #include "error.h"
 #include "errwarn.h"
+#include "interpreter.h"
 #include "interpreter-private.h"
 #include "load-save.h"
 #include "ls-ascii-helper.h"
@@ -234,9 +236,57 @@ extract_keyword (std::istream& is, const char *keyword, const bool next_only)
 
 #define SUBSTRING_COMPARE_EQ(s, pos, n, t) (s.substr (pos, n) == (t))
 
+static octave_value
+load_inline_fcn (std::istream& is, const std::string& filename)
+{
+  int nargs;
+  if (extract_keyword (is, "nargs", nargs, true))
+    {
+      std::string name;
+      octave_value_list args (nargs+1);
+      for (int i = 0; i < nargs; i++)
+        {
+          std::string tmp;
+          is >> tmp;
+          args(i+1) = tmp;
+        }
+      is >> name;
+      if (name == "0")
+        name = "";
+
+      skip_preceeding_newline (is);
+
+      std::string buf;
+
+      if (is)
+        {
+
+          // Get a line of text whitespace characters included,
+          // leaving newline in the stream.
+          buf = read_until_newline (is, true);
+        }
+
+      if (is)
+        {
+          args(0) = std::string (buf);
+
+          octave::interpreter& interp
+            = octave::__get_interpreter__ ("load_inline_fcn");
+
+          octave_value_list tmp = interp.feval ("inline", args, 1);
+
+          if (tmp.length () > 0)
+            return tmp(0);
+        }
+    }
+
+  error ("load: trouble reading ascii file '%s'", filename.c_str ());
+}
+
 std::string
 read_text_data (std::istream& is, const std::string& filename, bool& global,
-                octave_value& tc, octave_idx_type count)
+                octave_value& tc, octave_idx_type count,
+                const bool do_name_validation)
 {
   // Read name for this entry or break on EOF.
 
@@ -251,8 +301,8 @@ read_text_data (std::istream& is, const std::string& filename, bool& global,
       return "";
     }
 
-  if (! (name == ".nargin." || name == ".nargout."
-         || name == CELL_ELT_TAG || octave::valid_identifier (name)))
+  if (name != CELL_ELT_TAG
+      && do_name_validation && ! octave::valid_identifier (name))
     error ("load: invalid identifier '%s' found in file '%s'",
            name.c_str (), filename.c_str ());
 
@@ -278,6 +328,12 @@ read_text_data (std::istream& is, const std::string& filename, bool& global,
   // Special case for backward compatibility.  A small bit of cruft
   if (SUBSTRING_COMPARE_EQ (typ, 0, 12, "string array"))
     tc = charMatrix ();
+  else if (SUBSTRING_COMPARE_EQ (typ, 0, 15, "inline function"))
+    {
+      // Special case for loading old octave_inline_fcn objects.
+      tc = load_inline_fcn (is, filename);
+      return name;
+    }
   else
     {
       octave::type_info& type_info
