@@ -1,24 +1,27 @@
-/*
-
-Copyright (C) 2006-2018 John W. Eaton
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 2006-2021 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -40,7 +43,6 @@ along with Octave; see the file COPYING.  If not, see
 #include "quit.h"
 
 #include "Cell.h"
-#include "call-stack.h"
 #include "error.h"
 #include "interpreter-private.h"
 #include "interpreter.h"
@@ -50,6 +52,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "oct-map.h"
 #include "ovl.h"
 #include "ov.h"
+#include "ov-classdef.h"
 #include "ov-mex-fcn.h"
 #include "ov-usr-fcn.h"
 #include "pager.h"
@@ -669,7 +672,9 @@ class mxArray_matlab : public mxArray_base
 protected:
 
   mxArray_matlab (mxClassID id_arg = mxUNKNOWN_CLASS)
-    : mxArray_base (), class_name (nullptr), id (id_arg), ndims (0), dims (nullptr) { }
+    : mxArray_base (), class_name (nullptr), id (id_arg), ndims (0),
+      dims (nullptr)
+  { }
 
   mxArray_matlab (mxClassID id_arg, mwSize ndims_arg, const mwSize *dims_arg)
     : mxArray_base (), class_name (nullptr), id (id_arg),
@@ -1163,7 +1168,7 @@ public:
         for (size_t i = 0; i < tmp_len; i++)
           cpr[m*i+j] = static_cast<mxChar> (ptr[i]);
 
-        for (size_t i = tmp_len; i < static_cast<size_t>(nc); i++)
+        for (size_t i = tmp_len; i < static_cast<size_t> (nc); i++)
           cpr[m*i+j] = static_cast<mxChar> (' ');
       }
   }
@@ -1490,12 +1495,15 @@ public:
 
   mxArray_sparse (mxClassID id_arg, mwSize m, mwSize n, mwSize nzmax_arg,
                   mxComplexity flag = mxREAL)
-    : mxArray_matlab (id_arg, m, n), nzmax (nzmax_arg),
-      pr (mxArray::calloc (nzmax, get_element_size ())),
-      pi (flag == mxCOMPLEX ? mxArray::calloc (nzmax, get_element_size ()) : nullptr),
-      ir (static_cast<mwIndex *> (mxArray::calloc (nzmax, sizeof (mwIndex)))),
-      jc (static_cast<mwIndex *> (mxArray::calloc (n + 1, sizeof (mwIndex))))
-  { }
+    : mxArray_matlab (id_arg, m, n)
+  {
+    nzmax = (nzmax_arg > 0 ? nzmax_arg : 1);
+    pr = mxArray::calloc (nzmax, get_element_size ());
+    pi = (flag == mxCOMPLEX ? mxArray::calloc (nzmax, get_element_size ())
+                            : nullptr);
+    ir = (static_cast<mwIndex *> (mxArray::calloc (nzmax, sizeof (mwIndex))));
+    jc = (static_cast<mwIndex *> (mxArray::calloc (n + 1, sizeof (mwIndex))));
+  }
 
 private:
 
@@ -1560,7 +1568,11 @@ public:
 
   void set_jc (mwIndex *jc_arg) { jc = jc_arg; }
 
-  void set_nzmax (mwSize nzmax_arg) { nzmax = nzmax_arg; }
+  void set_nzmax (mwSize nzmax_arg)
+  {
+    /* Require storage for at least 1 element */
+    nzmax = (nzmax_arg > 0 ? nzmax_arg : 1);
+  }
 
   octave_value as_octave_value (void) const
   {
@@ -1890,7 +1902,7 @@ public:
 
     string_vector keys (fields, nfields);
 
-    octave_map m;
+    octave_map m (dv);
 
     mwSize ntot = nfields * get_number_of_elements ();
 
@@ -2132,7 +2144,7 @@ public:
     // We can't use mex::free here because it modifies memlist.
     while (! memlist.empty ())
       {
-        std::set<void *>::iterator p = memlist.begin ();
+        auto p = memlist.begin ();
         xfree (*p);
         memlist.erase (p);
       }
@@ -2140,7 +2152,7 @@ public:
     // We can't use mex::free_value here because it modifies arraylist.
     while (! arraylist.empty ())
       {
-        std::set<mxArray *>::iterator p = arraylist.begin ();
+        auto p = arraylist.begin ();
         delete *p;
         arraylist.erase (p);
       }
@@ -2155,10 +2167,10 @@ public:
   {
     if (! fname)
       {
-        octave::call_stack& cs
-          = octave::__get_call_stack__ ("mex::function_name");
+        octave::tree_evaluator& tw
+          = octave::__get_evaluator__ ("mex::function_name");
 
-        octave_function *fcn = cs.current ();
+        octave_function *fcn = tw.current_function ();
 
         if (fcn)
           {
@@ -2180,7 +2192,7 @@ public:
     if (! ptr)
       {
         // FIXME: could use "octave_new_handler();" instead
-        error ("%s: failed to allocate %d bytes of memory",
+        error ("%s: failed to allocate %zd bytes of memory",
                function_name (), n);
       }
 
@@ -2228,22 +2240,24 @@ public:
 
     if (ptr)
       {
+        auto p_local = memlist.find (ptr);
+        auto p_global = global_memlist.find (ptr);
+
         v = std::realloc (ptr, n);
 
-        std::set<void *>::iterator p = memlist.find (ptr);
-
-        if (v && p != memlist.end ())
+        if (v)
           {
-            memlist.erase (p);
-            memlist.insert (v);
-          }
+            if (p_local != memlist.end ())
+              {
+                memlist.erase (p_local);
+                memlist.insert (v);
+              }
 
-        p = global_memlist.find (ptr);
-
-        if (v && p != global_memlist.end ())
-          {
-            global_memlist.erase (p);
-            global_memlist.insert (v);
+            if (p_global != global_memlist.end ())
+              {
+                global_memlist.erase (p_global);
+                global_memlist.insert (v);
+              }
           }
       }
     else
@@ -2259,7 +2273,7 @@ public:
       {
         unmark (ptr);
 
-        std::set<void *>::iterator p = global_memlist.find (ptr);
+        auto p = global_memlist.find (ptr);
 
         if (p != global_memlist.end ())
           {
@@ -2296,7 +2310,7 @@ public:
   // made persistent, or because it was already freed.
   void unmark (void *ptr)
   {
-    std::set<void *>::iterator p = memlist.find (ptr);
+    auto p = memlist.find (ptr);
 
     if (p != memlist.end ())
       memlist.erase (p);
@@ -2314,7 +2328,7 @@ public:
 
   void unmark_array (mxArray *ptr)
   {
-    std::set<mxArray *>::iterator p = arraylist.find (ptr);
+    auto p = arraylist.find (ptr);
 
     if (p != arraylist.end ())
       arraylist.erase (p);
@@ -2334,7 +2348,7 @@ public:
   // Unmark a pointer as one we allocated.
   void unmark_foreign (void *ptr)
   {
-    std::set<void *>::iterator p = foreign_memlist.find (ptr);
+    auto p = foreign_memlist.find (ptr);
 
     if (p != foreign_memlist.end ())
       foreign_memlist.erase (p);
@@ -2357,7 +2371,7 @@ public:
   {
     bool inlist = false;
 
-    std::set<mxArray *>::iterator p = arraylist.find (ptr);
+    auto p = arraylist.find (ptr);
 
     if (p != arraylist.end ())
       {
@@ -2416,7 +2430,7 @@ private:
   // Unmark a pointer as one we allocated.
   void global_unmark (void *ptr)
   {
-    std::set<void *>::iterator p = global_memlist.find (ptr);
+    auto p = global_memlist.find (ptr);
 
     if (p != global_memlist.end ())
       global_memlist.erase (p);
@@ -3232,19 +3246,35 @@ mexCallMATLAB (int nargout, mxArray *argout[], int nargin,
   for (int i = 0; i < nargin; i++)
     args(i) = mxArray::as_octave_value (argin[i]);
 
+  octave::interpreter& interp = octave::__get_interpreter__ ("mexCallMATLAB");
+
   bool execution_error = false;
 
   octave_value_list retval;
 
   try
     {
+      octave::tree_evaluator& tw = interp.get_evaluator ();
+
+      octave::unwind_action act
+        ([&tw] (const std::list<octave::octave_lvalue> *lvl)
+         {
+           tw.set_lvalue_list (lvl);
+         }, tw.lvalue_list ());
+
+      tw.set_lvalue_list (nullptr);
+
       retval = octave::feval (fname, args, nargout);
     }
   catch (const octave::execution_exception&)
     {
       if (mex_context->trap_feval_error)
         {
-          octave::interpreter::recover_from_exception ();
+          // FIXME: is there a way to indicate what error occurred?
+          // Should the error message be displayed here?  Do we need to
+          // save the exception info for lasterror?
+
+          interp.recover_from_exception ();
 
           execution_error = true;
         }
@@ -3313,6 +3343,8 @@ mexEvalString (const char *s)
 {
   int retval = 0;
 
+  octave::interpreter& interp = octave::__get_interpreter__ ("mexEvalString");
+
   int parse_status;
   bool execution_error = false;
 
@@ -3320,11 +3352,11 @@ mexEvalString (const char *s)
 
   try
     {
-      ret = octave::eval_string (s, false, parse_status, 0);
+      ret = interp.eval_string (std::string (s), false, parse_status, 0);
     }
   catch (const octave::execution_exception&)
     {
-      octave::interpreter::recover_from_exception ();
+      interp.recover_from_exception ();
 
       execution_error = true;
     }
@@ -3340,6 +3372,8 @@ mexEvalStringWithTrap (const char *s)
 {
   mxArray *mx = nullptr;
 
+  octave::interpreter& interp = octave::__get_interpreter__ ("mexEvalString");
+
   int parse_status;
   bool execution_error = false;
 
@@ -3347,11 +3381,11 @@ mexEvalStringWithTrap (const char *s)
 
   try
     {
-      ret = octave::eval_string (s, false, parse_status, 0);
+      ret = interp.eval_string (std::string (s), false, parse_status, 0);
     }
   catch (const octave::execution_exception&)
     {
-      octave::interpreter::recover_from_exception ();
+      interp.recover_from_exception ();
 
       execution_error = true;
     }
@@ -3374,8 +3408,18 @@ mexEvalStringWithTrap (const char *s)
 void
 mexErrMsgTxt (const char *s)
 {
-  if (s && strlen (s) > 0)
-    error ("%s: %s", mexFunctionName (), s);
+  size_t len;
+
+  if (s && (len = strlen (s)) > 0)
+    {
+      if (s[len - 1] == '\n')
+        {
+          std::string s_tmp (s, len - 1);
+          error ("%s: %s\n", mexFunctionName (), s_tmp.c_str ());
+        }
+      else
+        error ("%s: %s", mexFunctionName (), s);
+    }
   else
     {
       // For compatibility with Matlab, print an empty message.
@@ -3409,7 +3453,24 @@ mexErrMsgIdAndTxt (const char *id, const char *fmt, ...)
 void
 mexWarnMsgTxt (const char *s)
 {
-  warning ("%s", s);
+  size_t len;
+
+  if (s && (len = strlen (s)) > 0)
+    {
+      if (s[len - 1] == '\n')
+        {
+          std::string s_tmp (s, len - 1);
+          warning ("%s\n", s_tmp.c_str ());
+        }
+      else
+        warning ("%s", s);
+    }
+  else
+    {
+      // For compatibility with Matlab, print an empty message.
+      // Octave's warning routine requires a non-null input so use a SPACE.
+      warning (" ");
+    }
 }
 
 void
@@ -3437,7 +3498,7 @@ mexPrintf (const char *fmt, ...)
   int retval;
   va_list args;
   va_start (args, fmt);
-  retval = octave_vformat (octave_stdout, fmt, args);
+  retval = octave::vformat (octave_stdout, fmt, args);
   va_end (args);
   return retval;
 }
@@ -3449,13 +3510,10 @@ mexGetVariable (const char *space, const char *name)
 
   octave_value val;
 
-  if (! strcmp (space, "global"))
-    {
-      octave::symbol_table& symtab
-        = octave::__get_symbol_table__ ("mexGetVariable");
+  octave::interpreter& interp = octave::__get_interpreter__ ("mexGetVariable");
 
-      val = symtab.global_varval (name);
-    }
+  if (! strcmp (space, "global"))
+    val = interp.global_varval (name);
   else
     {
       // FIXME: should this be in variables.cc?
@@ -3472,18 +3530,15 @@ mexGetVariable (const char *space, const char *name)
 
           if (base)
             {
-              octave::call_stack& cs
-                = octave::__get_call_stack__ ("mexGetVariable");
+              octave::tree_evaluator& tw = interp.get_evaluator ();
 
-              cs.goto_base_frame ();
+              frame.add_method (tw, &octave::tree_evaluator::restore_frame,
+                                tw.current_call_stack_frame_number ());
 
-              frame.add_method (cs, &octave::call_stack::pop);
+              tw.goto_base_frame ();
             }
 
-          octave::symbol_scope scope
-            = octave::__require_current_scope__ ("mexGetVariable");
-
-          val = scope.varval (name);
+          val = interp.varval (name);
         }
       else
         mexErrMsgTxt ("mexGetVariable: symbol table does not exist");
@@ -3520,13 +3575,10 @@ mexPutVariable (const char *space, const char *name, const mxArray *ptr)
   if (! name || name[0] == '\0')
     return 1;
 
-  if (! strcmp (space, "global"))
-    {
-      octave::symbol_table& symtab
-        = octave::__get_symbol_table__ ("mexPutVariable");
+  octave::interpreter& interp = octave::__get_interpreter__ ("mexPutVariable");
 
-      symtab.global_assign (name, mxArray::as_octave_value (ptr));
-    }
+  if (! strcmp (space, "global"))
+    interp.global_assign (name, mxArray::as_octave_value (ptr));
   else
     {
       // FIXME: should this be in variables.cc?
@@ -3543,18 +3595,15 @@ mexPutVariable (const char *space, const char *name, const mxArray *ptr)
 
           if (base)
             {
-              octave::call_stack& cs
-                = octave::__get_call_stack__ ("mexPutVariable");
+              octave::tree_evaluator& tw = interp.get_evaluator ();
 
-              cs.goto_base_frame ();
+              frame.add_method (tw, &octave::tree_evaluator::restore_frame,
+                                tw.current_call_stack_frame_number ());
 
-              frame.add_method (cs, &octave::call_stack::pop);
+              tw.goto_base_frame ();
             }
 
-          octave::symbol_scope scope
-            = octave::__require_current_scope__ ("mexPutVariable");
-
-          scope.assign (name, mxArray::as_octave_value (ptr));
+          interp.assign (name, mxArray::as_octave_value (ptr));
         }
       else
         mexErrMsgTxt ("mexPutVariable: symbol table does not exist");
@@ -3649,9 +3698,9 @@ mexLock (void)
 int
 mexSet (double handle, const char *property, mxArray *val)
 {
-  bool ret =
-    set_property_in_handle (handle, property, mxArray::as_octave_value (val),
-                            "mexSet");
+  bool ret
+    = set_property_in_handle (handle, property, mxArray::as_octave_value (val),
+                              "mexSet");
   return (ret ? 0 : 1);
 }
 
@@ -3662,7 +3711,7 @@ mexUnlock (void)
     {
       const char *fname = mexFunctionName ();
 
-      std::map<std::string,int>::iterator p = mex_lock_count.find (fname);
+      auto p = mex_lock_count.find (fname);
 
       if (p != mex_lock_count.end ())
         {
