@@ -1,24 +1,27 @@
-/*
-
-Copyright (C) 1996-2019 John W. Eaton
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1996-2021 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if ! defined (octave_oct_stdstrm_h)
 #define octave_oct_stdstrm_h 1
@@ -40,9 +43,10 @@ public:
                        std::ios::openmode m = std::ios::in | std::ios::out,
                        octave::mach_info::float_format ff
                          = octave::mach_info::native_float_format (),
+                       const std::string& encoding = "utf-8",
                        typename BUF_T::close_fcn cf = BUF_T::file_close)
-    : octave::base_stream (m, ff), nm (n), md (m),
-      s (f ? new STREAM_T (f, cf) : nullptr), fnum (fid)
+    : octave::base_stream (m, ff, encoding), m_name (n), m_mode (m),
+      m_stream (f ? new STREAM_T (f, cf) : nullptr), fnum (fid)
   { }
 
   // No copying!
@@ -54,48 +58,66 @@ public:
   // Position a stream at OFFSET relative to ORIGIN.
 
   int seek (off_t offset, int origin)
-  { return s ? s->seek (offset, origin) : -1; }
+  {
+    return m_stream ? m_stream->seek (offset, origin) : -1;
+  }
 
   // Return current stream position.
 
-  off_t tell (void) { return s ? s->tell () : -1; }
+  off_t tell (void) { return m_stream ? m_stream->tell () : -1; }
 
   // Return nonzero if EOF has been reached on this stream.
 
-  bool eof (void) const { return s ? s->eof () : true; }
+  bool eof (void) const { return m_stream ? m_stream->eof () : true; }
 
   // The name of the file.
 
-  std::string name (void) const { return nm; }
+  std::string name (void) const { return m_name; }
 
-  std::istream * input_stream (void) { return (md & std::ios::in) ? s : nullptr; }
+  std::istream * input_stream (void)
+  {
+    return (m_mode & std::ios::in) ? m_stream : nullptr;
+  }
 
-  std::ostream * output_stream (void) { return (md & std::ios::out) ? s : nullptr; }
+  std::ostream * output_stream (void)
+  {
+    return (m_mode & std::ios::out) ? m_stream : nullptr;
+  }
 
   // FIXME: should not have to cast away const here.
   BUF_T * rdbuf (void) const
-  { return s ? (const_cast<STREAM_T *> (s))->rdbuf () : nullptr; }
+  {
+    return m_stream ? (const_cast<STREAM_T *> (m_stream))->rdbuf () : nullptr;
+  }
 
   int file_number (void) const { return fnum; }
 
-  bool bad (void) const { return s ? s->bad () : true; }
+  bool bad (void) const { return m_stream ? m_stream->bad () : true; }
 
-  void clear (void) { if (s) s->clear (); }
+  void clear (void)
+  {
+    if (m_stream)
+      m_stream->clear ();
+  }
 
-  void do_close (void) { if (s) s->stream_close (); }
+  void do_close (void)
+  {
+    if (m_stream)
+      m_stream->stream_close ();
+  }
 
 protected:
 
-  std::string nm;
+  std::string m_name;
 
-  std::ios::openmode md;
+  std::ios::openmode m_mode;
 
-  STREAM_T *s;
+  STREAM_T *m_stream;
 
   // The file number associated with this file.
   int fnum;
 
-  ~octave_tstdiostream (void) { delete s; }
+  ~octave_tstdiostream (void) { delete m_stream; }
 };
 
 class
@@ -108,18 +130,20 @@ public:
                       std::ios::openmode m = std::ios::in | std::ios::out,
                       octave::mach_info::float_format ff
                         = octave::mach_info::native_float_format (),
+                      const std::string& encoding = "utf-8",
                       c_file_ptr_buf::close_fcn cf = c_file_ptr_buf::file_close)
     : octave_tstdiostream<c_file_ptr_buf, io_c_file_ptr_stream, FILE *>
-       (n, f, f ? fileno (f) : -1, m, ff, cf) { }
+       (n, f, f ? fileno (f) : -1, m, ff, encoding, cf) { }
 
   static octave::stream
   create (const std::string& n, FILE *f = nullptr,
           std::ios::openmode m = std::ios::in | std::ios::out,
           octave::mach_info::float_format ff
             = octave::mach_info::native_float_format (),
+          const std::string& encoding = "utf-8",
           c_file_ptr_buf::close_fcn cf = c_file_ptr_buf::file_close)
   {
-    return octave::stream (new octave_stdiostream (n, f, m, ff, cf));
+    return octave::stream (new octave_stdiostream (n, f, m, ff, encoding, cf));
   }
 
   // No copying!
@@ -145,19 +169,22 @@ public:
                        std::ios::openmode m = std::ios::in | std::ios::out,
                        octave::mach_info::float_format ff
                          = octave::mach_info::native_float_format (),
+                       const std::string& encoding = "utf-8",
                        c_zfile_ptr_buf::close_fcn cf
                          = c_zfile_ptr_buf::file_close)
     : octave_tstdiostream<c_zfile_ptr_buf, io_c_zfile_ptr_stream, gzFile>
-       (n, f, fid, m, ff, cf) { }
+       (n, f, fid, m, ff, encoding, cf) { }
 
   static octave::stream
   create (const std::string& n, gzFile f = nullptr, int fid = 0,
           std::ios::openmode m = std::ios::in | std::ios::out,
           octave::mach_info::float_format ff
             = octave::mach_info::native_float_format (),
+          const std::string& encoding = "utf-8",
           c_zfile_ptr_buf::close_fcn cf = c_zfile_ptr_buf::file_close)
   {
-    return octave::stream (new octave_zstdiostream (n, f, fid, m, ff, cf));
+    return octave::stream (new octave_zstdiostream (n, f, fid, m, ff, encoding,
+                                                    cf));
   }
 
   // No copying!
