@@ -1,25 +1,27 @@
-/*
-
-Copyright (C) 2013-2018 Vytautas Jančauskas
-Copyright (C) 2016-2018 Damjan Angelovski
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 2013-2021 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -98,9 +100,9 @@ is stored in the audio file.
 
   frame.add_fcn (safe_close, file);
 
-  OCTAVE_LOCAL_BUFFER (float, data, info.frames * info.channels);
+  OCTAVE_LOCAL_BUFFER (double, data, info.frames * info.channels);
 
-  sf_read_float (file, data, info.frames * info.channels);
+  sf_read_double (file, data, info.frames * info.channels);
 
   sf_count_t start = 0;
   sf_count_t end = info.frames;
@@ -153,13 +155,13 @@ is stored in the audio file.
           switch (info.format & SF_FORMAT_SUBMASK)
             {
             case SF_FORMAT_PCM_S8:
-              ret_audio = int8NDArray (audio * 127);
+              ret_audio = int8NDArray (audio * 128);
               break;
             case SF_FORMAT_PCM_U8:
-              ret_audio = uint8NDArray (audio * 127 + 127);
+              ret_audio = uint8NDArray (audio * 128 + 128);
               break;
             case SF_FORMAT_PCM_16:
-              ret_audio = int16NDArray (audio * 32767);
+              ret_audio = int16NDArray (audio * 32768);
               break;
             case SF_FORMAT_PCM_24:
               ret_audio = int32NDArray (audio * 8388608);
@@ -167,6 +169,9 @@ is stored in the audio file.
             case SF_FORMAT_PCM_32:
               ret_audio = int32NDArray (audio * 2147483648);
               break;
+            case SF_FORMAT_FLOAT:
+              ret_audio = FloatNDArray (audio);
+              break;
             default:
               ret_audio = audio;
               break;
@@ -285,19 +290,21 @@ Comment.
   double scale = 1.0;
 
   if (args(1).is_uint8_type ())
-    bias = scale = std::pow (2.0, 7);
+    bias = scale = 127.5;
   else if (args(1).is_int16_type ())
-    scale = std::pow (2.0, 15);
+    scale = 32768;       // 2^15
   else if (args(1).is_int32_type ())
-    scale = std::pow (2.0, 31);
+    scale = 2147483648;  // 2^31
   else if (args(1).isinteger ())
     err_wrong_type_arg ("audiowrite", args(1));
 
   Matrix audio = args(1).matrix_value ();
 
-  // FIXME: SampleRate is supposed to be a positive scalar greater than 0
-  //        Need check for that, and possibly convert to uint.
+  if (! args(2).is_scalar_type () || ! args(2).isnumeric ())
+    error ("audiowrite: sample rate FS must be a positive scalar integer");
   int samplerate = args(2).int_value ();
+  if (samplerate < 1)
+    error ("audiowrite: sample rate FS must be a positive scalar integer");
 
   std::string ext;
   size_t dotpos = filename.find_last_of ('.');
@@ -310,7 +317,7 @@ Comment.
   if (audio.rows () == 1)
     audio = audio.transpose ();
 
-  OCTAVE_LOCAL_BUFFER (float, data, items_to_write);
+  OCTAVE_LOCAL_BUFFER (double, data, items_to_write);
 
   sf_count_t idx = 0;
   for (int i = 0; i < audio.rows (); i++)
@@ -332,18 +339,17 @@ Comment.
     {
       info.format = SF_FORMAT_VORBIS;
 
-      // FIXME: there seems to be a bug writing ogg files in one shot
-      // that causes a segfault.  Breaking it up into a series of
-      // smaller chunks seems to avoid the problem and produce valid
-      // files.
-      chunk_size = 0x1FFFFE;
+      // FIXME: There seems to be a bug writing ogg files in one shot that
+      // causes a segfault: https://bugs.debian.org/760898.
+      // Breaking it up into a series of smaller chunks appears to avoid the
+      // problem and produces valid files.
+      chunk_size = 0x100000;
     }
   else
     info.format = SF_FORMAT_PCM_16;
 
   info.channels = audio.columns ();
   info.samplerate = samplerate;
-  info.channels = audio.cols ();
   info.format |= extension_to_format (ext);
 
   std::string title = "";
@@ -374,7 +380,12 @@ Comment.
           else if (bits == 16)
             info.format |= SF_FORMAT_PCM_16;
           else if (bits == 24)
-            info.format |= SF_FORMAT_PCM_32;
+            {
+              if ((info.format & SF_FORMAT_TYPEMASK) == SF_FORMAT_WAV)
+                info.format |= SF_FORMAT_PCM_32;
+              else
+                info.format |= SF_FORMAT_PCM_24;
+            }
           else if (bits == 32)
             {
               if ((info.format & SF_FORMAT_TYPEMASK) == SF_FORMAT_WAV
@@ -424,6 +435,8 @@ Comment.
 
   frame.add_fcn (safe_close, file);
 
+  sf_command (file, SFC_SET_NORM_DOUBLE, nullptr, SF_TRUE);
+  sf_command (file, SFC_SET_CLIPPING, nullptr, SF_TRUE) ;
   sf_command (file, SFC_SET_VBR_ENCODING_QUALITY, &quality, sizeof (quality));
 
   if (title != "")
@@ -446,11 +459,11 @@ Comment.
       if (items_to_write - offset < chunk_size)
         chunk_size = items_to_write - offset;
 
-      sf_count_t items_written = sf_write_float (file, data+offset, chunk_size);
+      sf_count_t items_written = sf_write_double (file, data+offset, chunk_size);
 
       if (items_written != chunk_size)
-        error ("audiowrite: write failed, wrote %ld of %ld items\n",
-               items_written, chunk_size);
+        error ("audiowrite: write failed, wrote %" PRId64 " of %" PRId64
+               " items\n", items_written, chunk_size);
 
       total_items_written += items_written;
       offset += chunk_size;
@@ -471,10 +484,75 @@ Comment.
 }
 
 /*
+## Joint audiowrite/audioread tests
+## 8-bit Unsigned PCM
+%!testif HAVE_SNDFILE <*56889>
+%! fname = [tempname() ".wav"];
+%! unwind_protect
+%!   y1 = uint8 ([0, 1, 2, 253, 254, 255]);
+%!   audiowrite (fname, y1, 8000, "BitsPerSample", 8);
+%!   y2 = audioread (fname, "native");
+%! unwind_protect_cleanup
+%!   unlink (fname);
+%! end_unwind_protect
+%! assert (y1(:), y2);
+
+## 8-bit Signed PCM
+%!testif HAVE_SNDFILE <*56889>
+%! fname = [tempname() ".au"];
+%! unwind_protect
+%!   y1 = uint8 ([0, 1, 2, 253, 254, 255]);
+%!   audiowrite (fname, y1, 8000, "BitsPerSample", 8);
+%!   y2 = audioread (fname, "native");
+%! unwind_protect_cleanup
+%!   unlink (fname);
+%! end_unwind_protect
+%! assert (y2, int8 ([-128; -127; -126; 125; 126; 127]));
+
+## 16-bit Signed PCM
+%!testif HAVE_SNDFILE <*56889>
+%! fname = [tempname() ".wav"];
+%! unwind_protect
+%!   y1 = int16 ([-32768, -32767, -32766, 32765, 32766, 32767]);
+%!   audiowrite (fname, y1, 8000, "BitsPerSample", 16);
+%!   y2 = audioread (fname, "native");
+%! unwind_protect_cleanup
+%!   unlink (fname);
+%! end_unwind_protect
+%! assert (y1(:), y2);
+
+## 24-bit Signed PCM
+%!testif HAVE_SNDFILE <*56889>
+%! fname = [tempname() ".au"];
+%! unwind_protect
+%!   y1 = [-8388608, -8388607, -8388606, 8388605, 8388606, 8388607] / 8388608;
+%!   audiowrite (fname, y1, 8000, "BitsPerSample", 24);
+%!   y2 = audioread (fname, "native");
+%! unwind_protect_cleanup
+%!   unlink (fname);
+%! end_unwind_protect
+%! assert (int32 ([-8388608; -8388607; -8388606; 8388605; 8388606; 8388607]),
+%!         y2);
+
+## 32-bit Signed PCM
+%!testif HAVE_SNDFILE <*56889>
+%! fname = [tempname() ".wav"];
+%! unwind_protect
+%!   y1 = int32 ([-2147483648, -2147483647, -2147483646, 2147483645, 2147483646, 2147483647 ]);
+%!   audiowrite (fname, y1, 8000, "BitsPerSample", 32);
+%!   y2 = audioread (fname, "native");
+%! unwind_protect_cleanup
+%!   unlink (fname);
+%! end_unwind_protect
+%! assert (y1(:), y2);
+
 ## Test input validation
 %!testif HAVE_SNDFILE
 %! fail ("audiowrite (1, 1, 8e3)", "FILENAME must be a string");
 %! fail ("audiowrite ('foo', int64 (1), 8e3)", "wrong type argument 'int64 scalar'");
+%! fail ("audiowrite ('foo', [0 1], [8e3, 8e3])", "FS must be a positive scalar");
+%! fail ("audiowrite ('foo', 1, {8e3})", "FS must be a .* integer");
+%! fail ("audiowrite ('foo', 1, -8e3)", "FS must be a positive");
 %! fail ("audiowrite ('foo', 1, 8e3, 'bitspersample')", "invalid number of arguments");
 %! fail ("audiowrite ('foo', 1, 8e3, 'bitspersample', 48)", "wrong number of bits specified");
 %! fail ("audiowrite ('foo', 1, 8e3, 'quality', [2 3 4])", "Quality value must be a scalar");
@@ -584,6 +662,12 @@ Audio bit rate.  Unused, only present for compatibility with @sc{matlab}.
     case SF_FORMAT_PCM_32:
       bits = 32;
       break;
+    case SF_FORMAT_FLOAT:
+      bits = 32;
+      break;
+    case SF_FORMAT_DOUBLE:
+      bits = 64;
+      break;
     default:
       bits = -1;
       break;
