@@ -1,24 +1,27 @@
-/*
-
-Copyright (C) 1996-2019 John W. Eaton
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1996-2021 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -143,6 +146,9 @@ octave_struct::subsref (const std::string& type,
 
             const Cell t = tmp.index (idx.front ());
 
+            // Avoid creating a comma-separated list if the result is a
+            // single element.
+
             retval(0) = (t.numel () == 1) ? t(0) : octave_value (t, true);
 
             // We handled two index elements, so tell
@@ -157,12 +163,12 @@ octave_struct::subsref (const std::string& type,
 
     case '.':
       {
-        if (map.numel () > 0)
-          {
-            const Cell t = dotref (idx.front ());
+        const Cell t = dotref (idx.front ());
 
-            retval(0) = (t.numel () == 1) ? t(0) : octave_value (t, true);
-          }
+        // Avoid creating a comma-separated list if the result is a
+        // single element.
+
+        retval(0) = (t.numel () == 1) ? t(0) : octave_value (t, true);
       }
       break;
 
@@ -206,6 +212,9 @@ octave_struct::subsref (const std::string& type,
 
             const Cell t = tmp.index (idx.front (), auto_add);
 
+            // Avoid creating a comma-separated list if the result is a
+            // single element.
+
             retval = (t.numel () == 1) ? t(0) : octave_value (t, true);
 
             // We handled two index elements, so tell
@@ -224,6 +233,9 @@ octave_struct::subsref (const std::string& type,
           {
             const Cell t = dotref (idx.front (), auto_add);
 
+            // Avoid creating a comma-separated list if the result is a
+            // single element.
+
             retval = (t.numel () == 1) ? t(0) : octave_value (t, true);
           }
       }
@@ -446,8 +458,7 @@ octave_struct::subsasgn (const std::string& type,
               }
             else
               {
-                const octave_map& cmap =
-                  const_cast<const octave_map &> (map);
+                const octave_map& cmap = const_cast<const octave_map &> (map);
                 // cast to const reference, avoid forced key insertion.
                 if (idxf.all_scalars ()
                     || cmap.contents (key).index (idxf, true).numel () == 1)
@@ -539,7 +550,10 @@ octave_value
 octave_struct::do_index_op (const octave_value_list& idx, bool resize_ok)
 {
   if (idx.length () == 0)
-    return map;
+    {
+      warn_empty_index (type_name ());
+      return map;
+    }
   else  // octave_map handles indexing itself.
     return map.index (idx, resize_ok);
 }
@@ -766,8 +780,7 @@ octave_struct::load_ascii (std::istream& is)
           bool dummy;
 
           // recurse to read cell elements
-          std::string nm
-            = read_text_data (is, "", dummy, t2, j);
+          std::string nm = read_text_data (is, "", dummy, t2, j, false);
 
           if (! is)
             break;
@@ -791,7 +804,7 @@ octave_struct::load_ascii (std::istream& is)
 }
 
 bool
-octave_struct::save_binary (std::ostream& os, bool& save_as_floats)
+octave_struct::save_binary (std::ostream& os, bool save_as_floats)
 {
   octave_map m = map_value ();
 
@@ -1429,7 +1442,7 @@ octave_scalar_struct::load_ascii (std::istream& is)
 
           // recurse to read cell elements
           std::string nm
-            = read_text_data (is, "", dummy, t2, j);
+            = read_text_data (is, "", dummy, t2, j, false);
 
           if (! is)
             break;
@@ -1451,7 +1464,7 @@ octave_scalar_struct::load_ascii (std::istream& is)
 }
 
 bool
-octave_scalar_struct::save_binary (std::ostream& os, bool& save_as_floats)
+octave_scalar_struct::save_binary (std::ostream& os, bool save_as_floats)
 {
   octave_map m = map_value ();
 
