@@ -1,25 +1,27 @@
-/*
-
-Copyright (C) 1996-2019 John W. Eaton
-Copyright (C) 2009-2010 VZLU Prague
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1996-2021 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -73,7 +75,6 @@ along with Octave; see the file COPYING.  If not, see
 #include "ov-dld-fcn.h"
 #include "ov-usr-fcn.h"
 #include "ov-fcn-handle.h"
-#include "ov-fcn-inline.h"
 #include "ov-typeinfo.h"
 #include "ov-null-mat.h"
 #include "ov-lazy-idx.h"
@@ -1106,10 +1107,6 @@ octave_value::octave_value (const octave_scalar_map& m, const std::string& id,
   : rep (new octave_class (m, id, plist))
 { }
 
-octave_value::octave_value (const octave_value_list& l, bool)
-  : rep (new octave_cs_list (l))
-{ }
-
 octave_value::octave_value (const octave_value_list& l)
   : rep (new octave_cs_list (l))
 { }
@@ -1696,12 +1693,6 @@ octave_value::fcn_handle_value (bool silent) const
   return rep->fcn_handle_value (silent);
 }
 
-octave_fcn_inline *
-octave_value::fcn_inline_value (bool silent) const
-{
-  return rep->fcn_inline_value (silent);
-}
-
 octave_value_list
 octave_value::list_value (void) const
 {
@@ -2121,7 +2112,6 @@ XVALUE_EXTRACTOR (octave_user_function *, xuser_function_value, user_function_va
 XVALUE_EXTRACTOR (octave_user_script *, xuser_script_value, user_script_value)
 XVALUE_EXTRACTOR (octave_user_code *, xuser_code_value, user_code_value)
 XVALUE_EXTRACTOR (octave_fcn_handle *, xfcn_handle_value, fcn_handle_value)
-XVALUE_EXTRACTOR (octave_fcn_inline *, xfcn_inline_value, fcn_inline_value)
 
 XVALUE_EXTRACTOR (octave_value_list, xlist_value, list_value)
 
@@ -2217,12 +2207,12 @@ do_binary_op (octave::type_info& ti, octave_value::binary_op op,
       else
         {
           octave_value tv1;
-          octave_base_value::type_conv_info cf1 =
-            v1.numeric_conversion_function ();
+          octave_base_value::type_conv_info cf1
+            = v1.numeric_conversion_function ();
 
           octave_value tv2;
-          octave_base_value::type_conv_info cf2 =
-            v2.numeric_conversion_function ();
+          octave_base_value::type_conv_info cf2
+            = v2.numeric_conversion_function ();
 
           // Try biased (one-sided) conversions first.
           if (cf2.type_id () >= 0
@@ -2554,6 +2544,16 @@ do_colon_op (const octave_value& base, const octave_value& increment,
       bool result_is_str = (base.is_string () && limit.is_string ());
       bool dq_str = (base.is_dq_string () || limit.is_dq_string ());
 
+      if (base.numel () > 1 || limit.numel () > 1
+          || (increment.is_defined () && increment.numel () > 1))
+        warning_with_id ("Octave:colon-nonscalar-argument",
+                         "colon arguments should be scalars");
+
+      if (base.iscomplex () || limit.iscomplex ()
+          || (increment.is_defined () && increment.iscomplex ()))
+        warning_with_id ("Octave:colon-complex-argument",
+                         "imaginary part of complex colon arguments is ignored");
+
       Matrix m_base, m_limit, m_increment;
 
       try
@@ -2954,7 +2954,6 @@ install_types (octave::type_info& ti)
   octave_user_function::register_type (ti);
   octave_dld_function::register_type (ti);
   octave_fcn_handle::register_type (ti);
-  octave_fcn_inline::register_type (ti);
   octave_float_scalar::register_type (ti);
   octave_float_complex::register_type (ti);
   octave_float_matrix::register_type (ti);
