@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 1996-2020 The Octave Project Developers
+// Copyright (C) 1996-2021 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -29,6 +29,8 @@
 
 #include "error.h"
 #include "errwarn.h"
+#include "interpreter-private.h"
+#include "interpreter.h"
 #include "ovl.h"
 #include "oct-lvalue.h"
 #include "ov.h"
@@ -235,6 +237,11 @@ namespace octave
         // could happen because there is an index is out of range
         // and we will be resizing a cell array.
 
+        interpreter& interp
+          = __get_interpreter__ ("octave_lvalue::eval_for_numel");
+
+        interp.recover_from_exception ();
+
         retval = octave_value ();
       }
 
