@@ -1,25 +1,27 @@
-/*
-
-Copyright (C) 2004-2019 John W. Eaton
-Copyright (C) 2008-2009 Jaroslav Hajek
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 2004-2021 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if ! defined (octave_oct_inttypes_h)
 #define octave_oct_inttypes_h 1
@@ -138,12 +140,13 @@ class octave_int_cmp_op
     static const bool t1sig = std::numeric_limits<T1>::is_signed;
     static const bool t2sig = std::numeric_limits<T2>::is_signed;
 
-    static const bool psig =
-      (pint || (sizeof (T2) > sizeof (T1) && t2sig) || t1sig);
+    static const bool psig
+      = (pint || (sizeof (T2) > sizeof (T1) && t2sig) || t1sig);
 
-    static const int psize =
-      (pint ? sizeof (int) : (sizeof (T2) > sizeof (T1)
-                              ? sizeof (T2) : sizeof (T1)));
+    static const int psize
+      = (pint
+         ? sizeof (int)
+         : (sizeof (T2) > sizeof (T1) ? sizeof (T2) : sizeof (T1)));
   public:
 
     typedef typename query_integer_type<psize, psig>::type type;
@@ -212,7 +215,7 @@ public:
   // We also provide two special relations: ct, yielding always true,
   // and cf, yielding always false.
 
-#define OCTAVE_REGISTER_INT_CONST_OP(NM, VALUE)     \
+#define OCTAVE_REGISTER_INT_CONST_OP(NM, VALUE) \
   class NM                                      \
   {                                             \
   public:                                       \
@@ -221,7 +224,7 @@ public:
     static const bool gtval = VALUE;            \
                                                 \
     template <typename T>                       \
-    static bool op (T, T) { return VALUE; }     \
+      static bool op (T, T) { return VALUE; }   \
   }
 
   OCTAVE_REGISTER_INT_CONST_OP (ct, true);
@@ -371,12 +374,12 @@ public:
     static const int t_size = sizeof (T);
     static const int s_size = sizeof (S);
 
-    static const bool omit_chk_min =
-      (! s_is_signed || (t_is_signed && t_size >= s_size));
+    static const bool omit_chk_min
+      = (! s_is_signed || (t_is_signed && t_size >= s_size));
 
-    static const bool omit_chk_max =
-      (t_size > s_size || (t_size == s_size
-                           && (! t_is_signed || s_is_signed)));
+    static const bool omit_chk_max
+      = (t_size > s_size
+         || (t_size == s_size && (! t_is_signed || s_is_signed)));
 
     // If the check can be omitted, substitute constant false
     // relation.
@@ -645,7 +648,7 @@ public:
                ? octave_int_base<T>::max_val ()
                : x - y)
             : (x < octave_int_base<T>::min_val () + y
-                ? octave_int_base<T>::min_val ()
+               ? octave_int_base<T>::min_val ()
                : x - y));
   }
 
@@ -794,58 +797,56 @@ public:
 
   typedef T val_type;
 
-  octave_int (void) : ival () { }
+  octave_int (void) : m_ival () { }
 
-  octave_int (T i) : ival (i) { }
+  octave_int (T i) : m_ival (i) { }
 
 #if defined (OCTAVE_HAVE_OVERLOAD_CHAR_INT8_TYPES)
 
   // Always treat characters as unsigned.
   octave_int (char c)
-    : ival (octave_int_base<T>::truncate_int (static_cast<unsigned char> (c)))
+    : m_ival (octave_int_base<T>::truncate_int (static_cast<unsigned char> (c)))
   { }
 
 #endif
 
   octave_int (double d)
-    : ival (octave_int_base<T>::convert_real (d)) { }
+    : m_ival (octave_int_base<T>::convert_real (d)) { }
 
   octave_int (float d)
-    : ival (octave_int_base<T>::convert_real (d)) { }
+    : m_ival (octave_int_base<T>::convert_real (d)) { }
 
 #if defined (OCTAVE_INT_USE_LONG_DOUBLE)
 
   octave_int (long double d)
-    : ival (octave_int_base<T>::convert_real (d)) { }
+    : m_ival (octave_int_base<T>::convert_real (d)) { }
 
 #endif
 
-  octave_int (bool b) : ival (b) { }
+  octave_int (bool b) : m_ival (b) { }
 
   template <typename U>
   octave_int (const U& i)
-    : ival(octave_int_base<T>::truncate_int (i)) { }
+    : m_ival(octave_int_base<T>::truncate_int (i)) { }
 
   template <typename U>
   octave_int (const octave_int<U>& i)
-    : ival (octave_int_base<T>::truncate_int (i.value ())) { }
+    : m_ival (octave_int_base<T>::truncate_int (i.value ())) { }
 
-  octave_int (const octave_int<T>& i) : ival (i.ival) { }
+  octave_int (const octave_int<T>&) = default;
 
-  octave_int& operator = (const octave_int<T>& i)
-  {
-    ival = i.ival;
-    return *this;
-  }
+  octave_int& operator = (const octave_int<T>&) = default;
 
-  T value (void) const { return ival; }
+  ~octave_int (void) = default;
+
+  T value (void) const { return m_ival; }
 
   const unsigned char * iptr (void) const
   {
-    return reinterpret_cast<const unsigned char *> (& ival);
+    return reinterpret_cast<const unsigned char *> (& m_ival);
   }
 
-  bool operator ! (void) const { return ! ival; }
+  bool operator ! (void) const { return ! m_ival; }
 
   bool bool_value (void) const { return static_cast<bool> (value ()); }
 
@@ -864,7 +865,7 @@ public:
   inline octave_int<T>                          \
   OPNAME () const                               \
   {                                             \
-    return octave_int_arith<T>::NAME (ival);    \
+    return octave_int_arith<T>::NAME (m_ival);  \
   }
 
   OCTAVE_INT_UN_OP (operator -, minus)
@@ -873,19 +874,19 @@ public:
 
 #undef OCTAVE_INT_UN_OP
 
-// Homogeneous binary integer operations.
-#define OCTAVE_INT_BIN_OP(OP, NAME, ARGT)       \
-  inline octave_int<T>                          \
-  operator OP (const ARGT& y) const             \
-  {                                             \
-    return octave_int_arith<T>::NAME (ival, y); \
-  }                                             \
-                                                \
-  inline octave_int<T>&                         \
-  operator OP##= (const ARGT& y)                \
-  {                                             \
-    ival = octave_int_arith<T>::NAME (ival, y); \
-    return *this;                               \
+  // Homogeneous binary integer operations.
+#define OCTAVE_INT_BIN_OP(OP, NAME, ARGT)               \
+  inline octave_int<T>                                  \
+  operator OP (const ARGT& y) const                     \
+  {                                                     \
+    return octave_int_arith<T>::NAME (m_ival, y);       \
+  }                                                     \
+                                                        \
+  inline octave_int<T>&                                 \
+  operator OP##= (const ARGT& y)                        \
+  {                                                     \
+    m_ival = octave_int_arith<T>::NAME (m_ival, y);     \
+    return *this;                                       \
   }
 
   OCTAVE_INT_BIN_OP (+, add, octave_int<T>)
@@ -912,11 +913,11 @@ public:
 
   // Unsafe.  This function exists to support the MEX interface.
   // You should not use it anywhere else.
-  void * mex_get_data (void) const { return const_cast<T *> (&ival); }
+  void * mex_get_data (void) const { return const_cast<T *> (&m_ival); }
 
 private:
 
-  T ival;
+  T m_ival;
 };
 
 template <typename T>
