@@ -1,20 +1,27 @@
-## Copyright (C) 1999-2017 David M. Doolin
+########################################################################
+##
+## Copyright (C) 1999-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {} polyarea (@var{x}, @var{y})
@@ -32,22 +39,13 @@
 ##
 ## @end deftypefn
 
-## todo:  Add moments for centroid, etc.
+## FIXME: Add moments for centroid, etc.
 ##
-## bugs and limitations:
+## Bugs and limitations:
 ##        Probably ought to be an optional check to make sure that
 ##        traversing the vertices doesn't make any sides cross
 ##        (Is simple closed curve the technical definition of this?).
 
-## Author: David M. Doolin <doolin@ce.berkeley.edu>
-## Date: 1999-04-14
-## Modified-by:
-##    2000-01-15 Paul Kienzle <pkienzle@kienzle.powernet.co.uk>
-##    * use matlab compatible interface
-##    * return absolute value of area so traversal order doesn't matter
-##    2005-10-13 Torsten Finke
-##    * optimization saving half the sums and multiplies
-
 function a = polyarea (x, y, dim)
 
   if (nargin != 2 && nargin != 3)
