@@ -1,4 +1,9 @@
-## Copyright (C) 2010-2019 Ben Abbott
+########################################################################
+##
+## Copyright (C) 2010-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {} rmappdata (@var{h}, @var{name})
@@ -28,9 +35,6 @@
 ## @seealso{setappdata, getappdata, isappdata}
 ## @end deftypefn
 
-## Author: Ben Abbott <bpabbott@mac.com>
-## Created: 2010-07-15
-
 function rmappdata (h, varargin)
 
   if (nargin < 2)
