@@ -1,5 +1,9 @@
-## Copyright (C) 2007-2019 Regents of the University of California
-## Copyright (C) 2016-2019 Marco Caliari
+########################################################################
+##
+## Copyright (C) 2007-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -16,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {@var{cest} =} condest (@var{A})
@@ -89,13 +95,13 @@
 ## @item
 ## @nospell{N.J. Higham and F. Tisseur}, @cite{A Block Algorithm
 ## for Matrix 1-Norm Estimation, with an Application to 1-Norm
-## Pseudospectra}. SIMAX vol 21, no 4, pp 1185--1201.
+## Pseudospectra}.  SIMAX vol 21, no 4, pp 1185--1201.
 ## @url{https://dx.doi.org/10.1137/S0895479899356080}
 ##
 ## @item
 ## @nospell{N.J. Higham and F. Tisseur}, @cite{A Block Algorithm
 ## for Matrix 1-Norm Estimation, with an Application to 1-Norm
-## Pseudospectra}. @url{https://citeseer.ist.psu.edu/223007.html}
+## Pseudospectra}.  @url{https://citeseer.ist.psu.edu/223007.html}
 ## @end itemize
 ##
 ## @seealso{cond, norm, normest1, normest}
@@ -202,7 +208,7 @@ function [cest, v] = condest (varargin)
   warning ("off", "Octave:nearly-singular-matrix", "local");
 
   if (! have_solve_normest1)
-     ## prepare solve in normest1 form
+    ## prepare solve in normest1 form
     if (issparse (A))
       [L, U, P, Pc] = lu (A);
       solve = @(flag, x) solve_sparse (flag, x, n, real_op, L, U, P, Pc);
@@ -234,15 +240,18 @@ function [cest, v] = condest (varargin)
 endfunction
 
 function value = solve_sparse (flag, x, n, real_op, L , U , P , Pc)
+  ## FIXME: Sparse algorithm is less accurate than full matrix version
+  ##        See BIST test for non-orthogonal matrix where relative tolerance
+  ##        of 1e-12 is used for sparse, but 4e-16 for full matrix.
   switch (flag)
     case "dim"
       value = n;
     case "real"
       value = real_op;
     case "notransp"
-      value = P' * (L' \ (U' \ (Pc * x)));
-    case "transp"
       value = Pc' * (U \ (L \ (P * x)));
+    case "transp"
+      value = P' * (L' \ (U' \ (Pc * x)));
   endswitch
 endfunction
 
@@ -253,9 +262,9 @@ function value = solve_not_sparse (flag, x, n, real_op, L, U, P)
     case "real"
       value = real_op;
     case "notransp"
-      value = P' * (L' \ (U' \ x));
-    case "transp"
       value = U \ (L \ (P * x));
+    case "transp"
+      value = P' * (L' \ (U' \ x));
   endswitch
 endfunction
 
@@ -333,6 +342,7 @@ endfunction
 %! cA_test = norm (inv (A^2), 1) * norm (A^2, 1);
 %! assert (cA, cA_test, -2^-6);
 
+## Test singular matrices
 %!test <*46737>
 %! A = [ 0         0         0
 %!       0   3.33333 0.0833333
@@ -341,6 +351,19 @@ endfunction
 %! assert (cest, Inf);
 %! assert (v, []);
 
+## Test non-orthogonal matrices
+%!test <*57968>
+%! A = reshape (sqrt (0:15), 4, 4);
+%! cexp = norm (A, 1) * norm (inv (A), 1);
+%! cest = condest (A);
+%! assert (cest, cexp, -2*eps);
+
+%!testif HAVE_UMFPACK <*57968>
+%! As = sparse (reshape (sqrt (0:15), 4, 4));
+%! cexp = norm (As, 1) * norm (inv (As), 1);
+%! cest = condest (As);
+%! assert (cest, cexp, -1e-12);
+
 ## Test input validation
 %!error condest ()
 %!error condest (1,2,3,4,5,6,7)
