@@ -1,36 +1,46 @@
-## Copyright (C) 1993-2017 Auburn University.  All rights reserved.
+########################################################################
+##
+## Copyright (C) 1993-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn {} {[@var{u}, @var{h}, @var{nu}] =} krylov (@var{A}, @var{V}, @var{k}, @var{eps1}, @var{pflg})
-## Construct an orthogonal basis @var{u} of block Krylov subspace
+## Construct an orthogonal basis @var{u} of a block Krylov subspace.
+##
+## The block Krylov subspace has the following form:
 ##
 ## @example
 ## [v a*v a^2*v @dots{} a^(k+1)*v]
 ## @end example
 ##
 ## @noindent
-## using Householder reflections to guard against loss of orthogonality.
+## The construction is made with Householder reflections to guard against loss
+## of orthogonality.
 ##
 ## If @var{V} is a vector, then @var{h} contains the Hessenberg matrix
 ## such that @nospell{@tcode{a*u == u*h+rk*ek'}}, in which
 ## @code{rk = a*u(:,k)-u*h(:,k)}, and @nospell{@tcode{ek'}} is the vector
-## @code{[0, 0, @dots{}, 1]} of length @code{k}.  Otherwise, @var{h} is
+## @code{[0, 0, @dots{}, 1]} of length @var{k}.  Otherwise, @var{h} is
 ## meaningless.
 ##
 ## If @var{V} is a vector and @var{k} is greater than @code{length (A) - 1},
@@ -53,9 +63,7 @@
 ## the 42nd IEEE Conference on Decision and Control, December 2003.
 ## @end deftypefn
 
-## Author: A. Scottedward Hodel <a.s.hodel@eng.auburn.edu>
-
-function [Uret, H, nu] = krylov (A, V, k, eps1, pflg);
+function [Uret, H, nu] = krylov (A, V, k, eps1, pflg)
 
   if (isa (A, "single") || isa (V, "single"))
     defeps = 1e-6;
@@ -165,7 +173,7 @@ function [Uret, H, nu] = krylov (A, V, k, eps1, pflg);
 
         ## Reduce V per the reflection.
         V(idx,:) = V(idx,:) - av*hv*(hv' * V(idx,:));
-        if(iter > 1)
+        if (iter > 1)
           ## FIXME: not done correctly for block case.
           H(nu,nu-1) = V(pivot_vec(nu),jj);
         endif
@@ -218,7 +226,7 @@ function [Uret, H, nu] = krylov (A, V, k, eps1, pflg);
   ## Back out complete U matrix.
   ## back out U matrix.
   j1 = columns (U);
-  for i = j1:-1:1;
+  for i = j1:-1:1
     idx = pivot_vec(i:na);
     hv = U(idx,i);
     av = alpha(i);
