@@ -1,4 +1,9 @@
-## Copyright (C) 2012-2019 Jordi Gutiérrez Hermoso
+########################################################################
+##
+## Copyright (C) 2012-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {} nthargout (@var{n}, @var{func}, @dots{})
@@ -64,8 +71,6 @@
 ## @seealso{nargin, nargout, varargin, varargout, isargout}
 ## @end deftypefn
 
-## Author: Jordi Gutiérrez Hermoso
-
 function out = nthargout (n, varargin)
 
   if (nargin < 2)
