@@ -1,25 +1,32 @@
-## Copyright (C) 2006-2015 Bill Denney
+########################################################################
+##
+## Copyright (C) 2006-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
-## @deftypefn  {Function File} {@var{files} =} unpack (@var{file})
-## @deftypefnx {Function File} {@var{files} =} unpack (@var{file}, @var{dir})
-## @deftypefnx {Function File} {@var{files} =} unpack (@var{file}, @var{dir}, @var{filetype})
+## @deftypefn  {} {@var{files} =} unpack (@var{file})
+## @deftypefnx {} {@var{files} =} unpack (@var{file}, @var{dir})
+## @deftypefnx {} {@var{files} =} unpack (@var{file}, @var{dir}, @var{filetype})
 ## Unpack the archive @var{file} based on its extension to the directory
 ## @var{dir}.
 ##
@@ -38,11 +45,11 @@
 ## Valid filetype extensions:
 ##
 ## @table @code
-## @item  bz
-## @itemx bz2
+## @item  @nospell{bz}
+## @itemx @nospell{bz2}
 ## bzip archive
 ##
-## @item gz
+## @item @nospell{gz}
 ## gzip archive
 ##
 ## @item tar
@@ -69,9 +76,7 @@
 ## @seealso{bunzip2, gunzip, unzip, untar, bzip2, gzip, zip, tar}
 ## @end deftypefn
 
-## Author: Bill Denney <denney@seas.upenn.edu>
-
-function filelist = unpack (file, dir = ".", filetype = "")
+function filelist = unpack (file, dir = [], filetype = "")
 
   if (nargin < 1 || nargin > 3)
     print_usage ();
@@ -86,11 +91,14 @@ function filelist = unpack (file, dir = ".", filetype = "")
     file = cellstr (file);
   endif
   if (numel (file) == 1)
-    gfile = glob (file);
-    if (isempty (gfile))
-      error ('unpack: file "%s" not found', file{1});
-    else
-      file = gfile;
+    ## FIXME: The code below is not a perfect test for a URL
+    if (isempty (strfind (file{1}, "://")))
+      gfile = glob (file);
+      if (isempty (gfile))
+        error ('unpack: FILE "%s" not found', file{1});
+      else
+        file = gfile;
+      endif
     endif
   endif
 
@@ -98,7 +106,11 @@ function filelist = unpack (file, dir = ".", filetype = "")
   if (numel (file) > 1)
     files = {};
     for i = 1:numel (file)
-      tmpfiles = unpack (file{i}, dir);
+      if (! isempty (dir))
+        tmpfiles = unpack (file{i}, dir);
+      else
+        tmpfiles = unpack (file{i}, fileparts (file{i}));
+      endif
       files = {files{:} tmpfiles{:}};
     endfor
 
@@ -112,16 +124,29 @@ function filelist = unpack (file, dir = ".", filetype = "")
     file = file{1};
   endif
 
-  if (isdir (file))
+  if (nargin == 3 && (! ischar (filetype) || ! isrow (filetype)))
+    error ("unpack: FILETYPE must be a string");
+  endif
+
+  if (isfolder (file))
     if (isempty (filetype))
       error ("unpack: FILETYPE must be given for a directory");
-    elseif (! any (strcmpi (filetype, "gunzip")))
-      error ('unpack: FILETYPE must be "gunzip" for a directory');
+    elseif (! strcmpi (filetype, "gz"))
+      error ('unpack: FILETYPE must be "gz" for a directory');
     endif
     ext = ".gz";
   else
     [pathstr, name, ext] = fileparts (file);
 
+    if (nargin == 3 && ! strcmpi (ext, filetype))
+      ## override extension with given filetype
+      if (isempty (ext))
+        ext = filetype;
+      else
+        ext = regexprep (ext, '(\.?)\S*$', ['$1' filetype]);
+      endif
+    endif
+
     ## Check to see if it's .tar.gz, .tar.Z, etc.
     if (any (strcmpi ({".gz" ".Z" ".bz2" ".bz"}, ext)))
       [~, tmpname, tmpext] = fileparts (name);
@@ -135,7 +160,7 @@ function filelist = unpack (file, dir = ".", filetype = "")
     if (! isempty (strfind (file, "://")))
       ## FIXME: The above code is not a perfect test for a URL
       urlfile = file;
-      tmpfile = [tempname ext];
+      tmpfile = fullfile (tempdir (), [name ext]);
       [file, success, msg] = urlwrite (urlfile, tmpfile);
       if (! success)
         error ('unpack: could not fetch "%s": %s', urlfile, msg);
@@ -190,7 +215,7 @@ function filelist = unpack (file, dir = ".", filetype = "")
   endif
 
   ## Unzip doesn't actually care about the extension
-  if (strcmpi (filetype, "unzip"))
+  if (strcmpi (filetype, "zip"))
     nodotext = "zip";
   else
     nodotext = ext(ext != '.');
@@ -201,6 +226,17 @@ function filelist = unpack (file, dir = ".", filetype = "")
     file = __w2mpth__ (file);
   endif
 
+  ## Create the output directory if necessary.
+  s = stat (dir);
+  if (isempty (s))
+    [status, msg] = mkdir (dir);
+    if (! status)
+      error ("unpack: mkdir failed to create %s: %s", dir, msg);
+    endif
+  elseif (! S_ISDIR (s.mode))
+    error ("unpack: %s: not a directory", dir);
+  endif
+
   if (isfield (commandlist, tolower (nodotext)))
     [commandv, commandq, parsefcn, move] = deal (commandlist.(nodotext){:});
     origdir = pwd ();
@@ -214,34 +250,29 @@ function filelist = unpack (file, dir = ".", filetype = "")
     if (cenddir(end) == filesep)
       cenddir(end) = [];
     endif
-    needmove = move && ! strcmp (cstartdir, cenddir);
+    needmove = move && ! is_same_file (cstartdir, cenddir);
     if (nargout > 0 || needmove)
       command = commandv;
     else
       command = commandq;
     endif
   else
-    warning ("unpack: unrecognized FILETYPE <%s>", ext);
-    files = file;
+    warning ("unpack: unrecognized FILETYPE <%s>", nodotext);
+    filelist = {};
     return;
   endif
 
-  ## Create the directory if necessary.
-  s = stat (dir);
-  if (isempty (s))
-    [status, msg] = mkdir (dir);
-    if (! status)
-      error ("unpack: mkdir failed to create %s: %s", dir, msg);
-    endif
-  elseif (! S_ISDIR (s.mode))
-    error ("unpack: %s: not a directory", dir);
-  endif
-
+  ## Save and restore the TAR_OPTIONS environment variable used by GNU tar.
+  tar_options_env = getenv ("TAR_OPTIONS");
   unwind_protect
+    unsetenv ("TAR_OPTIONS");
     cd (dir);
     [status, output] = system (sprintf ([command " 2>&1"], file));
   unwind_protect_cleanup
     cd (origdir);
+    if (! isempty (tar_options_env))
+      setenv ("TAR_OPTIONS", tar_options_env);
+    endif
   end_unwind_protect
 
   if (status)
@@ -278,79 +309,104 @@ function files = __parse_zip__ (output)
   ## Skip first line which is Archive header.
   files = char (output(2:end));
   ## Trim constant width prefix and return cell array.
-  files = cellstr (files(:,14:end))
+  files = cellstr (files(:,14:end));
 endfunction
 
 function output = __parse_tar__ (output)
-  ## This is a no-op, but it makes things simpler for other cases.
+  ## BSD tar emits file actions in the first 2 columns
+
+  if (tar_is_bsd ())
+    output = cellfun (@(x) x(3:end), output, 'UniformOutput', false);
+  endif
 endfunction
 
 function files = __parse_gzip__ (output)
   ## Parse the output from gzip and gunzip returning the files
-  ## commpressed (or decompressed).
+  ## compressed (or decompressed).
 
-  files = regexprep (output, '^.+ with (.*)$', '$1');
+  files = regexprep (output, '^.+ -- replaced with (.*)$', '$1');
 endfunction
 
 function files = __parse_bzip2__ (output)
   ## Parse the output from bzip2 and bunzip2 returning the files
-  ## commpressed (or decompressed).
+  ## compressed (or decompressed).
 
-  ## Strip leading blanks and .bz2 extension from file name
+  ## Strip leading blanks and .bz2 extension from filename
   files = regexprep (output, '^\s+(.*)\.bz2: .*', '$1');
 endfunction
 
 
-%!test
-%! ## Create temporary directory and file for packing and unpacking
-%! dirname = tempname ();
-%! assert (mkdir (dirname));
-%! filename = tempname ();
-%! fid = fopen (filename, "wt");
-%! assert (fid >= 0);
-%! fprintf (fid, "Hello World\n");
-%! fprintf (fid, "123 456 789\n");
-%! fclose (fid);
+%!testif HAVE_ZLIB
+%! envvar = {"TMPDIR", "TMP"};
+%! envdir = cellfun (@(x) getenv (x), envvar, "uniformoutput", false);
 %! unwind_protect
-%!   copyfile (filename, [filename ".orig"]);
-%!   gzip (filename, dirname);
-%!   [~, f] = fileparts (filename);
-%!   filelist = unpack (fullfile (dirname, [f ".gz"]), P_tmpdir);
-%!   assert (filelist{1}, filename);
-%!   fid = fopen ([filename ".orig"], "rb");
+%!   cellfun (@(x) unsetenv (x), envvar);
+%!   ## Create temporary directory and file for packing and unpacking
+%!   dirname = tempname ();
+%!   assert (mkdir (dirname));
+%!   filename = tempname ();
+%!   fid = fopen (filename, "wt");
 %!   assert (fid >= 0);
-%!   orig_data = fread (fid);
+%!   fprintf (fid, "Hello World\n");
+%!   fprintf (fid, "123 456 789\n");
 %!   fclose (fid);
-%!   fid = fopen (filename, "rb");
-%!   assert (fid >= 0);
-%!   new_data = fread (fid);
-%!   fclose (fid);
-%!   if (orig_data != new_data)
-%!     error ("unpack: Unpacked file does not equal original");
-%!   endif
+%!
+%!   unwind_protect
+%!     copyfile (filename, [filename ".orig"]);
+%!     gzip (filename, dirname);
+%!     [~, f] = fileparts (filename);
+%!     filelist = unpack (fullfile (dirname, [f ".gz"]), tempdir);
+%!     assert (filelist{1}, filename);
+%!     fid = fopen ([filename ".orig"], "rb");
+%!     assert (fid >= 0);
+%!     orig_data = fread (fid);
+%!     fclose (fid);
+%!     fid = fopen (filename, "rb");
+%!     assert (fid >= 0);
+%!     new_data = fread (fid);
+%!     fclose (fid);
+%!     if (orig_data != new_data)
+%!       error ("unpack: Unpacked file does not equal original");
+%!     endif
+%!   unwind_protect_cleanup
+%!     unlink (filename);
+%!     unlink ([filename ".orig"]);
+%!     confirm_recursive_rmdir (false, "local");
+%!     rmdir (dirname, "s");
+%!   end_unwind_protect
 %! unwind_protect_cleanup
-%!   unlink (filename);
-%!   unlink ([filename ".orig"]);
-%!   rmdir (dirname);
+%!   ## Restore environment variables TMPDIR, TMP
+%!   for i = 1:numel (envvar)
+%!     if (isempty (envdir{i}))
+%!       unsetenv (envvar{i});
+%!     else
+%!       setenv (envvar{i}, envdir{i});
+%!     endif
+%!   endfor
 %! end_unwind_protect
 
 ## Test input validation
 %!error unpack ()
 %!error unpack (1,2,3,4)
 %!error <FILE must be a string or cell array of strings> unpack (1)
-%!error <file "_%NOT_A_FILENAME%_" not found> unpack ("_%NOT_A_FILENAME%_")
-%!error <file "_%NOT_A_FILENAME%_" not found> unpack ({"_%NOT_A_FILENAME%_"})
-%!error <file "_%NOT_A_FILENAME%_" not found> unpack ({"_%NOT_A_FILENAME%_", "2nd_filename"})
+%!error <FILE "_%NOT_A_FILENAME%_" not found> unpack ("_%NOT_A_FILENAME%_")
+%!error <FILE "_%NOT_A_FILENAME%_" not found> unpack ({"_%NOT_A_FILENAME%_"})
+%!error <FILE "_%NOT_A_FILENAME%_" not found> unpack ({"_%NOT_A_FILENAME%_", "2nd_filename"})
+%!error <FILETYPE must be a string>
+%! if (isunix || ismac)
+%!   unpack ("/", [], 1)
+%! else
+%!   unpack ('C:\', [], 1)
+%! endif
 %!error <FILETYPE must be given for a directory>
 %! if (isunix || ismac)
 %!   unpack ("/");
 %! else
 %!   unpack ('C:\');
 %! endif
-%!error <FILETYPE must be "gunzip" for a directory>
+%!error <FILETYPE must be "gz" for a directory>
 %! if (isunix || ismac)
 %!   unpack ("/", [], "foobar");
 %! else
 %!   unpack ('C:\', [], "foobar");
 %! endif
-
