@@ -1,4 +1,9 @@
-## Copyright (C) 2016-2018 Markus Muetzel
+########################################################################
+##
+## Copyright (C) 2016-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {@var{reduced_fv} =} reducepatch (@var{fv})
@@ -77,8 +84,6 @@
 ## @seealso{isosurface, isonormals, reducevolume, patch}
 ## @end deftypefn
 
-## Author: mmuetzel
-
 ## FIXME: Convert faces to only triangles if necessary
 
 function varargout = reducepatch (varargin)
@@ -197,7 +202,7 @@ function [faces, vertices, max_faces, patch_handle, fast, verbose] = ...
              "FACES, second argument must be a matrix containing VERTICES"]);
     endif
   else
-    print_usage ();
+    print_usage ("reducepatch");
   endif
 
   ## get reduction_factor
@@ -317,7 +322,7 @@ function [faces_reduced, vertices_reduced] = ...
       vertices(end_point_idx,:) = Inf; # mark vertex for later removal
     endif
 
-    ## Pretty print a row of dots while performing calulation
+    ## Pretty print a row of dots while performing calculation
     if (verbose && ++verbose_counter > verbose_stepwidth)
       printf (".");
       verbose_counter = 0;
