@@ -1,4 +1,9 @@
-## Copyright (C) 1994-2018 John W. Eaton
+########################################################################
+##
+## Copyright (C) 1994-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,23 +20,31 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn {} {} shg
 ## Show the graph window.
 ##
-## Currently, this is the same as executing @code{drawnow}.
-## @seealso{drawnow, figure}
+## This function makes the current figure visible, and places it on top of
+## of all other plot windows.
+##
+## Programming Note: @code{shg} is equivalent to @code{figure (gcf)} assuming
+## that a current figure exists.
+## @seealso{figure, drawnow, gcf}
 ## @end deftypefn
 
-## Author: jwe
-
 function shg ()
 
   if (nargin != 0)
     warning ("shg: ignoring extra arguments");
   endif
 
-  drawnow ();
+  hf = get (0, "currentfigure");
+  if (! isempty (hf))
+    set (hf, "visible", "on");
+    __show_figure__ (hf);
+  endif
 
 endfunction
