@@ -1,21 +1,27 @@
-## Copyright (C) 2014-2017 Eduardo Ramos Fernández <eduradical951@gmail.com>
-## Copyright (C) 2013-2016 Kai T. Ohlhus <k.ohlhus@gmail.com>
+########################################################################
+##
+## Copyright (C) 2013-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {@var{L} =} ichol (@var{A})
@@ -152,9 +158,6 @@
 ## @seealso{chol, ilu, pcg}
 ## @end deftypefn
 
-## Author: Eduardo Ramos Fernández <eduradical951@gmail.com>
-## Author: Kai T. Ohlhus <k.ohlhus@gmail.com>
-
 function L = ichol (A, opts = struct ())
 
   if (nargin < 1 || nargin > 2 || nargout > 1)
