@@ -1,4 +1,9 @@
-## Copyright (C) 2000-2019 Paul Kienzle
+########################################################################
+##
+## Copyright (C) 2000-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {@var{pf} =} factor (@var{q})
@@ -34,8 +41,6 @@
 ## @seealso{gcd, lcm, isprime, primes}
 ## @end deftypefn
 
-## Author: Paul Kienzle
-
 function [pf, n] = factor (q)
 
   if (nargin != 1)
