@@ -1,4 +1,9 @@
-## Copyright (C) 1999-2019 Peter Ekberg
+########################################################################
+##
+## Copyright (C) 1999-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn {} {} rosser ()
@@ -24,9 +31,6 @@
 ## @seealso{wilkinson, eig}
 ## @end deftypefn
 
-## Author: Peter Ekberg
-##         (peda)
-
 function retval = rosser ()
 
   if (nargin != 0)
