@@ -1,20 +1,27 @@
-## Copyright (C) 2000-2017 Paul Kienzle
+########################################################################
+##
+## Copyright (C) 2000-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {} assert (@var{cond})
@@ -88,7 +95,7 @@ function assert (cond, varargin)
           || isempty (cond) || ! all (cond(:)))
         if (nargin == 1)
           ## Perhaps, say which elements failed?
-          argin = ["(" strjoin(cellstr (argn), ",") ")"];
+          argin = ["(" inputname(1, false) ")"];
           error ("assert %s failed", argin);
         else
           error (varargin{:});
@@ -181,27 +188,30 @@ function assert (cond, varargin)
           err.observed{end+1} = ["O(" sprintf("%dx", size(cond))(1:end-1) ")"];
           err.expected{end+1} = ["E(" sprintf("%dx", size(expected))(1:end-1) ")"];
           err.reason{end+1} = "Structure sizes don't match";
+        elseif (! strcmp (sort (fieldnames (cond)),
+                          sort (fieldnames (expected))))
+          err.index{end+1} = ".";
+          err.observed{end+1} = "O";
+          err.expected{end+1} = "E";
+          err.reason{end+1} = "Structure fieldname mismatch";
         else
           try
-            empty = isempty (cond);
-            normal = (numel (cond) == 1);
-            for [v, k] = cond
-              if (! isfield (expected, k))
-                err.index{end+1} = ".";
-                err.observed{end+1} = "O";
-                err.expected{end+1} = "E";
-                err.reason{end+1} = ["'" k "'" " is not an expected field"];
-              endif
-              if (empty)
-                v = {};
-              elseif (normal)
-                v = {v};
-              else
-                v = v(:)';
-              endif
-              ## Recursively call assert for struct array values
-              assert (v, {expected.(k)}, tol);
-            endfor
+            ## Test if both structs are empty, i.e. 0x0, Nx0, or Nx0 structs.
+            ## In this case the values cannot be extracted for comparison as
+            ## they are not assignable and not defined.
+            assert (isempty (cond), isempty (expected));
+
+            if (! isempty (cond))
+              for [v, k] = cond
+                if (numel (cond) == 1)
+                  v = {v};
+                else
+                  v = v(:)';
+                endif
+                ## Recursively call assert for struct array values
+                assert (v, {expected.(k)}, tol);
+              endfor
+            endif
           catch
             err.index{end+1} = ".";
             err.observed{end+1} = "O";
@@ -225,7 +235,7 @@ function assert (cond, varargin)
             err.observed{end+1} = "O";
             err.expected{end+1} = "E";
             err.reason{end+1} = ["Class " class(cond) " != " class(expected)];
-          elseif (isnumeric (cond))
+          elseif (isnumeric (cond) || islogical (cond))
             if (issparse (cond) != issparse (expected))
               err.index{end+1} = "()";
               err.observed{end+1} = "O";
@@ -333,8 +343,11 @@ function assert (cond, varargin)
             err.expected(end+1:end+length (erridx)) = ...
               strtrim (cellstr (num2str (B(erridx) (:))));
             err.reason(end+1:end+length (erridx)) = ...
-              ostrsplit (deblank (sprintf ("Abs err %.5g exceeds tol %.5g\n",...
-              [abs(A_null(erridx) - B_null(erridx))(:) mtol(erridx)(:)]')), "\n");
+              ostrsplit (deblank (
+                         sprintf ("Abs err %.5g exceeds tol %.5g by %.1g\n",
+                [abs(A_null(erridx) - B_null(erridx))(:), mtol(erridx)(:), ...
+                 abs(A_null(erridx) - B_null(erridx))(:)-mtol(erridx)(:)].')),
+                         "\n");
           endif
 
           k = (mtol > 0);
@@ -347,8 +360,11 @@ function assert (cond, varargin)
             err.expected(end+1:end+length (erridx)) = ...
               strtrim (cellstr (num2str (B(erridx) (:))));
             err.reason(end+1:end+length (erridx)) = ...
-              ostrsplit (deblank (sprintf ("Abs err %.5g exceeds tol %.5g\n",...
-              [abs(A_null(erridx) - B_null(erridx))(:) mtol(erridx)(:)]')), "\n");
+              ostrsplit (deblank (
+                         sprintf ("Abs err %.5g exceeds tol %.5g by %.1g\n",
+                [abs(A_null(erridx) - B_null(erridx))(:), mtol(erridx)(:), ...
+                 abs(A_null(erridx) - B_null(erridx))(:)-mtol(erridx)(:)].')),
+                         "\n");
           endif
 
           k = (mtol < 0);
@@ -363,8 +379,11 @@ function assert (cond, varargin)
               err.expected(end+1:end+length (erridx)) = ...
                 strtrim (cellstr (num2str (B(erridx) (:))));
               err.reason(end+1:end+length (erridx)) = ...
-                ostrsplit (deblank (sprintf ("Abs err %.5g exceeds tol %.5g\n",
-                [abs(A_null(erridx) - B_null(erridx)) -mtol(erridx)]')), "\n");
+                ostrsplit (deblank (
+                           sprintf ("Abs err %.5g exceeds tol %.5g by %.1g\n",
+                      [abs(A_null(erridx) - B_null(erridx)), -mtol(erridx), ...
+                       abs(A_null(erridx) - B_null(erridx))+mtol(erridx)].')),
+                           "\n");
             endif
             ## Test for relative error
             Bdiv = Inf (size (B_null));
@@ -379,8 +398,11 @@ function assert (cond, varargin)
               err.expected(end+1:end+length (erridx)) = ...
                 strtrim (cellstr (num2str (B(erridx) (:))));
               err.reason(end+1:end+length (erridx)) = ...
-                ostrsplit (deblank (sprintf ("Rel err %.5g exceeds tol %.5g\n",
-                [relerr(erridx)(:) -mtol(erridx)(:)]')), "\n");
+                ostrsplit (deblank (
+                           sprintf ("Rel err %.5g exceeds tol %.5g by %.1g\n",
+                                    [relerr(erridx)(:), -mtol(erridx)(:), ...
+                                     relerr(erridx)(:)+mtol(erridx)(:)].')),
+                           "\n");
             endif
           endif
         endif
@@ -389,7 +411,11 @@ function assert (cond, varargin)
 
       ## Print any errors
       if (! isempty (err.index))
-        argin = ["(" strjoin(cellstr (argn), ",") ")"];
+        arg_names = cell (nargin, 1);
+        for i = 1:nargin
+          arg_names{i} = inputname (i, false);
+        endfor
+        argin = ["(" strjoin(arg_names, ",") ")"];
         if (! isempty (errmsg))
           errmsg = [errmsg "\n"];
         endif
@@ -601,7 +627,7 @@ endfunction
 %! x.b = 1;
 %! y.a = 1;
 %! assert (x,y);
-%!error <'b' is not an expected field>
+%!error <Structure fieldname mismatch>
 %! x.b = 1;
 %! y.a = 1;
 %! assert (x,y);
@@ -624,7 +650,9 @@ endfunction
 %! x = [-40:0];
 %! y1 = (10.^x).*(10.^x);
 %! y2 = 10.^(2*x);
-%! assert (y1, y2, eps (y1));
+%! ## Increase tolerance from eps (y1) to 4*eps (y1) because of an upstream bug
+%! ## in mingw-w64: https://sourceforge.net/p/mingw-w64/bugs/466/
+%! assert (y1, y2, 4*eps (y1));
 %! fail ("assert (y1, y2 + eps*1e-70, eps (y1))");
 
 ## Multiple tolerances
@@ -645,6 +673,14 @@ endfunction
 %!   assert (! isempty (regexp (errmsg, '\(2,1\).*Rel err 2 exceeds tol 0.2')));
 %! end_try_catch
 
+%!test <*57615>
+%! try
+%!   assert (complex (pi*1e-17,2*pi), 0, 1e-1);
+%! catch
+%!   errmsg = lasterr ();
+%!   assert (isempty (strfind (errmsg, "sprintf: invalid field width")));
+%! end_try_catch
+
 ## test input validation
 %!error assert ()
 %!error assert (1,2,3,4)
@@ -678,10 +714,10 @@ function str = pprint (argin, err)
     leno = length (err.observed{i});
     lene = length (err.expected{i});
     str = [str, sprintf("%*s%*s %*s%*s %*s%*s   %s\n",
-                        6+fix(leni/2), err.index{i}   , 6-fix(leni/2), "",
-                        6+fix(leno/2), err.observed{i}, 6-fix(leno/2), "",
-                        6+fix(lene/2), err.expected{i}, 6-fix(lene/2), "",
-                        err.reason{i})];
+                  6+fix(leni/2), err.index{i}   , max (6-fix(leni/2), 0), "",
+                  6+fix(leno/2), err.observed{i}, max (6-fix(leno/2), 0), "",
+                  6+fix(lene/2), err.expected{i}, max (6-fix(lene/2), 0), "",
+                  err.reason{i})];
   endfor
 
 endfunction
